@@ -1,4 +1,4 @@
-Kmatrix <- function(model, modelterm, covariate=NULL, covariateV=NULL, prtnum=FALSE)
+Kmatrix <- function(model, modelterm, covariate=NULL, covariateV=NULL, data=NULL, prtnum=FALSE)
 {
   if (inherits(model, "mer") || inherits(model, "merMod")) { 
     if(!lme4::isLMM(model) && !lme4::isGLMM(model))
@@ -22,13 +22,15 @@ Kmatrix <- function(model, modelterm, covariate=NULL, covariateV=NULL, prtnum=FA
   # Get model formula and index of response
   Terms <- terms(model)
   yname <- as.character(attr(Terms, "variables"))[[2]]
+
   Terms <- delete.response(Terms)
   
   # get the pure formula w/o extra stuff
   formrhs <- formula(Terms)
-  
+ # if (random_term && inherits(model, "merMod")) formrhs <- formula(model)[-2]
   # All the variables in the model
   nm <- all.vars(formrhs)
+  
   nm <- nm[nm!="pi"]
   
   # Figure out if any are coerced to factor or ordered
@@ -40,8 +42,26 @@ Kmatrix <- function(model, modelterm, covariate=NULL, covariateV=NULL, prtnum=FA
   envir <- attr(Terms, ".Environment")
   #eval(thecall$data, envir=envir)
   
-  if (inherits(model, "mer") || inherits(model, "merMod") || inherits(model, "lme") || inherits(model, "gls")) model_data <- get_all_vars(formrhs, getData(model))
-    else model_data <- model.frame(model)
+  if (inherits(model, "mer") || inherits(model, "merMod") || inherits(model, "lme") || inherits(model, "gls")) {
+	model_data <- get_all_vars(formula(terms(model)), getData(model)) 
+    model_data <- droplevels(na.omit(model_data))
+    model_data <- get_all_vars(formrhs, model_data)
+    }else{
+      if(!is.null(data)) {
+	        if (grepl("[,]", yname)) {
+      yname <- unlist(strsplit(yname, "[,] "))[2]
+      yname <- gsub("\\)", "", unlist(strsplit(yname, " - "))) # ynames
+    }
+	  model_data <- as.data.frame(data[, c(yname, nm)]) 
+	  }else model_data <- model.frame(model)
+	}
+	
+	if (is.null(covariate) || !setequal(covariate, modelterm)) {
+  vars <- unlist(strsplit(modelterm, "\\:"))
+  if(any(!is.element(vars, names(model_data)[sapply(model_data, is.factor)])))
+    stop(paste(vars, "must be factor(s) or not in the fixed effects!"))
+	}
+	
   X <- model.frame(form, model_data, 
                    subset = eval(thecall$subset, enclos=envir),
                    na.action = na.omit, drop.unused.levels = TRUE)			  
@@ -68,18 +88,29 @@ Kmatrix <- function(model, modelterm, covariate=NULL, covariateV=NULL, prtnum=FA
   
   factor_names <- c(names(xlev), coerced)
   covlevname <- setdiff(names(baselevs), factor_names)
+
+  if (length(factor_names)!=0) {
   n.table <- table(preddf[, factor_names, drop = FALSE])
   ndf <- data.frame(n.table)
+  }else ndf <- NULL
   
-  if ((!is.null(covariate) && !unique(covariate%in%c("NULL", ""))) && is.numeric(covariate)) baselevs[covlevname] <- as.list(covariate)
-  if ((!is.null(covariate) && !unique(covariate%in%c("NULL", ""))) && is.character(covariate) && covariate%in%covlevname && length(covariate)==1)  {
+  if ((!is.null(covariate) && !unique(covariate%in%c("NULL", ""))) && all(is.numeric(covariate))) baselevs[covlevname] <- as.list(covariate)
+  if ((!is.null(covariate) && !unique(covariate%in%c("NULL", ""))) && is.character(covariate) && all(covariate%in%covlevname))  {
+   if (length(covariate)==1){
     # if (as.is) {
       # baselevs[[covariate]] <- sort(unique(X[[covariate]]))
     # }else{ 
       if ((!is.null(covariateV) && !unique(covariateV%in%c("NULL", ""))) && is.vector(covariateV)){
         baselevs[[covariate]] <- covariateV
-      }else baselevs[[covariate]] <- sort(unique(c(seq(min(X[[covariate]]), max(X[[covariate]]),length=500), X[[covariate]])))
+      }else baselevs[[covariate]] <- sort(unique(c(seq(min(X[[covariate]]), max(X[[covariate]]),length=50), X[[covariate]])))
    # }
+   }else{
+   if ((!is.null(covariateV) && !unique(covariateV%in%c("NULL", ""))) && is.matrix(covariateV)){
+        for (i in seq_along(covariate)) baselevs[[covariate[i]]] <- covariateV[,i]
+      }else{
+	    for (i in covariate) baselevs[[i]] <- sort(unique(c(seq(min(X[[i]]), max(X[[i]]),length=30))))  
+   }
+   }
   }
   if (all(length(covlevname)!=0, prtnum)) {
     cat("\n", "The predicted means are estimated at \n\n")
@@ -95,15 +126,25 @@ Kmatrix <- function(model, modelterm, covariate=NULL, covariateV=NULL, prtnum=FA
   
   # Now make a new dataset with just the factor combs and covariate values we want for prediction
   # WARNING -- This will overwrite X, so get anything you need from X BEFORE we get here
-  ndf <- merge( grid, ndf, sort=FALSE)
+  if (!is.null(ndf)) ndf <- merge( grid, ndf, sort=FALSE)
   
   m <- model.frame(Terms, grid, na.action = na.pass, xlev = xlev)
   X <- model.matrix(Terms, m, contrasts.arg = contrasts)
   
-  if (any(ndf$Freq==0)) {
+  # if (!is.null(ndf) && any(ndf$Freq==0)) {
+    # warning("Missing treatments' combination appeared, predicted means maybe misleading!")
+    # X[ndf$Freq==0, ] <- NA
+  # }
+  
+  max_ord <- max(attr(Terms,"order"))
+  if (!setequal(covariate, modelterm) && max_ord > 1 && length(factor_names) > 1  && any(ndf$Freq==0) && !(setequal(unique(ndf$Freq), c(0, 1)))) { #  && any(ndf$Freq==0)
+    
     warning("Missing treatments' combination appeared, predicted means maybe misleading!")
     X[ndf$Freq==0, ] <- NA
+   # X <- X[ndf$Freq!=0, ]
   }
+   
+  
   
   # All factors (excluding covariates)
   allFacs <- all.var.names
@@ -112,11 +153,11 @@ Kmatrix <- function(model, modelterm, covariate=NULL, covariateV=NULL, prtnum=FA
   row.indexes <- array(seq_len(nrow(X)), sapply(baselevs, length))  
   
   # convert a string to a formula
-  form <- as.formula(paste("~",modelterm))
+  form <- as.formula(paste("~", paste(modelterm, collapse = "+")))
   
   # These are the variables involved; and the label to use in the results
   facs <- all.vars(form)
-  if ((!is.null(covariate) && !unique(covariate%in%c("NULL", ""))) && all(is.character(covariate), !covariate%in%facs)) facs <- c(facs, covariate)   
+  if ((!is.null(covariate) && !unique(covariate%in%c("NULL", ""))) && all(is.character(covariate), all(!covariate%in%facs))) facs <- c(facs, covariate)   
   
   if (any(sapply(facs, function(nm) length(grep(nm, allFacs)) == 0)))
     stop(paste("Unknown factor(s) in specification:", paste(form, collapse=" ")))
@@ -141,5 +182,7 @@ Kmatrix <- function(model, modelterm, covariate=NULL, covariateV=NULL, prtnum=FA
   if (length(setdiff(colnames(model.matrix(model)), colnames(K)))!=0) stop(paste("You may need drop empty levels in factor", modelterm))
   K <- K[, colnames(model.matrix(model)), drop=FALSE]
   
+  fctnames <- cbind(fctnames, K[, setdiff(covlevname, names(fctnames)), drop=FALSE])
+  
   return(list(K=K, fctnames=fctnames, response=yname, preddf=preddf))
 }
